<?php
declare(strict_types=1);

namespace Inti\Wp;

use Mezzio\Authentication\AuthenticationInterface;
use Mezzio\Authentication\AuthenticationMiddleware;

/**
 * Class ConfigProvider
 *
 * @package Inti\Wp
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp
 */
final class ConfigProvider
{
    public function __invoke(): array
    {
        return [
            'wp' => [
                'connection' => 'default',

                'assets' => [
                    'manifest_path' => 'public/dist/manifest.json',
                    'dequeue' => [],
                ],

                'globals' => [],
                'plugins' => [],
                'shortcodes' => [],
            ],

            'wp_plugins' => [],
            'wp_shortcodes' => [],

            'dependencies' => $this->getDependencies(),
            'middleware_pipeline' => $this->getPipelineConfig(),
        ];
    }

    public function getDependencies(): array
    {
        return [
            'aliases' => [
                AuthenticationInterface::class => Authentication\WordPressAuthentication::class,

                'PluginLoader' => Plugin\PluginLoader::class,
                'PluginManager' => Plugin\PluginPluginManager::class,
                'ShortcodeLoader' => Shortcode\ShortcodeLoader::class,
                'ShortcodeManager' => Shortcode\ShortcodePluginManager::class,
            ],

            'factories' => [
                'assets' => Service\AssetManifestFactory::class,
                AuthenticationMiddleware::class => Authentication\AuthenticationMiddlewareFactory::class,

                Handler\Wp::class => Handler\WpFactory::class,
                Middleware\WpBootstrap::class => Middleware\WpBootstrapFactory::class,

                Plugin\PluginLoader::class => Plugin\PluginLoaderFactory::class,
                Plugin\PluginPluginManager::class => Plugin\PluginPluginManagerFactory::class,
                Shortcode\ShortcodeLoader::class => Shortcode\ShortcodeLoaderFactory::class,
                Shortcode\ShortcodePluginManager::class => Shortcode\ShortcodePluginManagerFactory::class,
            ],
        ];
    }

    public function getPipelineConfig(): array
    {
        return [[
            'middleware' => Handler\Wp::class,
            'priority' => 0,
        ]];
    }
}
