<?php
declare(strict_types=1);

namespace Inti\Wp\Middleware;

use Inti\Wp\Plugin\PluginLoader;
use Inti\Wp\Shortcode\ShortcodeLoader;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use function add_action;
use function define;
use function dirname;
use function rtrim;

/**
 * Class WordPressMiddleware
 *
 * @package Inti\Wp
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp
 */
final class WpBootstrap implements MiddlewareInterface
{
    /** @var array */
    private $config;
    /** @var bool */
    private static $bootstrapped = false;
    /** @var PluginLoader */
    private $pluginLoader;
    /** @var ShortcodeLoader */
    private $shortcodeLoader;

    /**
     * Konstruktor.
     *
     * @param array $config
     * @param PluginLoader $pluginLoader
     * @param ShortcodeLoader $shortcodeLoader
     */
    public function __construct(array $config, PluginLoader $pluginLoader, ShortcodeLoader $shortcodeLoader)
    {
        $this->config = $config;
        $this->pluginLoader = $pluginLoader;
        $this->shortcodeLoader = $shortcodeLoader;
    }

    /**
     * Process an incoming server request.
     *
     * Processes an incoming server request in order to produce a response.
     * If unable to produce the response itself, it may delegate to the provided
     * request handler to do so.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        if (self::$bootstrapped === false) {
            $this->bootstrap($request);
            self::$bootstrapped = true;
        }

        return $handler->handle($request);
    }

    /**
     * @param ServerRequestInterface $request
     * @noinspection DuplicatedCode
     */
    public function bootstrap(ServerRequestInterface $request): void
    {
        $dbConfig = $this->config['db_connections'][$this->config['wp']['connection']];

        define('DB_NAME', $dbConfig['dbname']);
        define('DB_USER', $dbConfig['user']);
        define('DB_PASSWORD', $dbConfig['password']);
        define('DB_HOST', $dbConfig['host']);
        define('DB_CHARSET', 'utf8mb4');
        define('DB_COLLATE', 'utf8mb4_croatian_ci');

        define('AUTH_KEY',         'put your unique phrase here');
        define('SECURE_AUTH_KEY',  'put your unique phrase here');
        define('LOGGED_IN_KEY',    'put your unique phrase here');
        define('NONCE_KEY',        'put your unique phrase here');
        define('AUTH_SALT',        'put your unique phrase here');
        define('SECURE_AUTH_SALT', 'put your unique phrase here');
        define('LOGGED_IN_SALT',   'put your unique phrase here');
        define('NONCE_SALT',       'put your unique phrase here');

        /** @noinspection PhpUnusedLocalVariableInspection */
        $table_prefix = 'wp_';

        define('DISABLE_WP_CRON', 'true');

        define('WP_DEBUG', $this->config['debug'] ?? false);

        $uri = $request->getUri();
        define('WP_HOME', rtrim((string) $uri->withQuery('')->withPath('/'), '/'));
        define('WP_SITEURL', WP_HOME . '/wp');
        define('WP_CONTENT_DIR', dirname(ABSPATH));
        define('WP_CONTENT_URL', WP_HOME);

        require_once ABSPATH . 'wp-includes/plugin.php';

        add_action('plugins_loaded', [$this->pluginLoader, 'attach']);
        add_action('after_setup_theme', [$this->shortcodeLoader, 'register']);

        foreach ($this->config['wp']['globals'] as $global) {
            global $$global;
        }

        require_once ABSPATH . 'wp-settings.php';
    }
}
