<?php
declare(strict_types=1);

namespace Inti\Wp\Plugin\Core;

use _WP_Dependency;
use Inti\Wp\Plugin\PluginInterface;
use WP_Scripts;
use function add_action;

/**
 * Class Assets
 *
 * @package Inti\Wp\Plugin\Core
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp
 */
final class Assets implements PluginInterface
{
    private array $manifest;
    private array $dequeue;

    public function __construct(array $manifest, array $dequeue)
    {
        $this->manifest = $manifest;
        $this->dequeue = $dequeue;
    }

    public function attach(): void
    {
        add_action('wp_default_scripts', [$this, 'wp_default_scripts'], PHP_INT_MAX);
        add_action('wp_print_footer_scripts', [$this, 'wp_dequeue_scripts'], 0);
        add_action('wp_print_scripts', [$this, 'wp_dequeue_scripts'], 0);
        add_action('wp_print_styles', [$this, 'wp_dequeue_scripts'], 0);
        add_action('wp_enqueue_scripts', [$this, 'fix_dependency_urls'], PHP_INT_MAX);
    }

    public function wp_default_scripts(WP_Scripts $scripts)
    {
        if (isset($this->manifest['jquery.js'])) {
            $scripts->registered['jquery'] = new _WP_Dependency('jquery', 'jquery.js', [], '3.5.1');
            unset($scripts->registered['jquery-core']);
            unset($scripts->registered['jquery-migrate']);
        }
    }

    public function wp_dequeue_scripts()
    {
        global $wp_scripts, $wp_styles;

        $wp_scripts->dequeue($this->dequeue);
        $wp_styles->dequeue($this->dequeue);
    }

    public function fix_dependency_urls()
    {
        global $wp_scripts, $wp_styles;

        foreach ([$wp_scripts, $wp_styles] as $dependencies) {
            foreach ($dependencies->registered as $dependency) {
                /** @var _WP_Dependency $dependency */
                if (isset($this->manifest[$dependency->src])) {
                    $dependency->src = WP_HOME . $this->manifest[$dependency->src];
                }
            }
        }
    }
}
