<?php
declare(strict_types=1);

namespace Inti\Wp\Shortcode;

use RuntimeException;
use function add_shortcode;

/**
 * Class ShortcodeLoader
 *
 * @package Inti\Wp\Shortcode
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp
 */
final class ShortcodeLoader
{
    /** @var string[] */
    private $tags;
    /** @var ShortcodePluginManager */
    private $shortcodes;

    /**
     * Konstruktor.
     *
     * @param string[] $tags
     * @param ShortcodePluginManager $shortcodes
     */
    public function __construct(array $tags, ShortcodePluginManager $shortcodes)
    {
        $this->tags = $tags;
        $this->shortcodes = $shortcodes;
    }

    public function register(): void
    {
        foreach ($this->tags as $tag) {
            add_shortcode($tag, [$this, 'render']);
        }
    }

    /**
     * @param array|string $atts
     * @param string $content
     * @param string $tag
     * @return string
     */
    public function render($atts, string $content = '', string $tag = ''): string
    {
        if (!$this->shortcodes->has($tag)) {
            throw new RuntimeException(sprintf(
                '%s: Invalid shortcode tag `%s`.', __METHOD__, $tag
            ));
        }

        /** @var ShortcodeInterface $shortcode */
        $shortcode = $this->shortcodes->get($tag);

        return $shortcode->render((array) $atts, $content);
    }
}
