const path = require('path');
const {CleanWebpackPlugin} = require('clean-webpack-plugin');
const CssMinimizerPlugin = require('css-minimizer-webpack-plugin');
const {WebpackManifestPlugin} = require('webpack-manifest-plugin');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const RemoveEmptyScriptsPlugin = require('webpack-remove-empty-scripts');
const TerserPlugin = require('terser-webpack-plugin');

// noinspection JSValidateTypes,JSUnresolvedVariable
module.exports = {

    entry: {
        'jquery': path.resolve(__dirname, 'assets/jquery.js'),
    },

    output: {
        filename: 'js/[name].min.js',
        path: path.resolve(__dirname, 'public/dist'),
        publicPath: '/dist/',
    },

    optimization: {
        minimize: true,
        minimizer: [new TerserPlugin({
            extractComments: false,
            terserOptions: {
                format: {
                    comments: false,
                }
            }
        })],
    },

    plugins: [
        new CleanWebpackPlugin(),
        new RemoveEmptyScriptsPlugin(),
        new WebpackManifestPlugin(),
        new CssMinimizerPlugin({
            minimizerOptions: {
                preset: ['default', {
                    discardComments: { removeAll: true },
                }],
            }
        }),
        new MiniCssExtractPlugin({
            filename: 'css/[name].css',
        }),
    ],

    module: {
        rules: [{
            test: /\.scss$/,
            use: [
                MiniCssExtractPlugin.loader,
                'css-loader',
                'sass-loader',
            ],
        }]
    },

    resolve: {
        alias: {
            'hello-elementor': path.resolve(__dirname, 'public/themes/hello-elementor'),
        }
    }
};
